#!/usr/bin/env python
"""
FUNCTION: Selects all PKS proteins (multi-hit ones with a high enough
          GPHS value and single-his ones that are placed in the PKS
          branch) from Sargasso Sea and uses the file
          'venter.contig_sample.mapping.txt' (produced by Christian
          von Merings) to allocated the source sample to it.

USAGE: PKS_db_analyse_Sargasso_Sea_hit_source_samples.py <DB_FILE> <MIN_GPHS> <CONTIG_SAMPLE_MAPPING_FILE>
         
Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import kuffileproc
import pks_db_tools

def main():
    if not len(sys.argv) == 4:
        print __doc__
        sys.exit(2)

    analyser = SGSSourceAnalyser(sys.argv[1])
    analyser.get_proteins(sys.argv[2])
    analyser.get_source_sample(sys.argv[3])
    analyser.print_output()
    
class SGSSourceAnalyser(object):

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()

    def get_proteins(self, min_GPHS):
        min_GPHS = float(min_GPHS)
        self.multi_domain_hit_proteins = {}
        self.single_domain_hit_proteins = []
        self.used_contigs = []
        for protein in self.db_connector.get_all_proteins_of_source('SGS'):
            # DEVELOPMENT ONLY####
            #if not protein.startswith('AACY01677517'): continue
            #if not protein.startswith('AACY0101'): continue
            ######################
            GPHS = self.db_connector.calc_GPHS(protein)
            if GPHS != None and GPHS > 0:
                self.multi_domain_hit_proteins[protein] = GPHS
            elif GPHS == None:
                self._process_single_hit_protein(protein)
            else:
                continue
            self.used_contigs.append(self._get_conting_from_protein_name(
                protein))

    def get_source_sample(self, sample_allocation_file):
        self.contigs_and_sources = {}
        for line in open(sample_allocation_file):
            if line[0] in ['#', '\n']: continue
            contig, sample, value = line[:-1].split()
            if contig in self.used_contigs:
                if self.contigs_and_sources.has_key(contig):
                    self.contigs_and_sources[contig].append([sample, value])
                else:
                    self.contigs_and_sources[contig] = [[sample, value]]

    def _get_conting_from_protein_name(self, protein):
        return(protein.split('.')[0])

    def _process_single_hit_protein(self, protein):
        hits = self.db_connector.get_all_hits_of_protein(protein)
        if len(hits) != 1:
            print ('Error: Something is fishy with protein "%s". '
                   'It should have exactly one domain hit.' %(protein))            
            return
        if self.db_connector.get_tree_based_annotation_of_hit(hits[0]) != 'PKS':
            return
        self.single_domain_hit_proteins.append(protein)

    def print_output(self):
        fileproc = kuffileproc.FileProcessor()
        output_file = fileproc.get_file('SGS_hits_source_samples.txt', 'w')
        
        output_file.write(
            '# Multi domain hit proteins\n' 
            '# Protein\tDomain structure\tGPHS\tSource samples\n')
        for protein, GPHS in self.multi_domain_hit_proteins.items():
            contig = self._get_conting_from_protein_name(protein)
            output_file.write('%s\t%s\t%s\t' % (
                protein,
                self.db_connector.build_domain_composition_string(protein),
                GPHS))
            if self.contigs_and_sources.has_key(contig):
                for sample, value in self.contigs_and_sources[contig]:
                    sample = sample.split('_') [-1]
                    output_file.write('sample_%s (%s) ' % (sample, value))
            else:
                output_file.write('no_sample_information')
            output_file.write('\n')

        output_file.write(
            '\n\n# Single domain hit proteins\n'
            '# Protein\tDomain structure\tSource samples\n')
        for protein in self.single_domain_hit_proteins:
            output_file.write('%s %s\t' % (
                protein,
                self.db_connector.build_domain_composition_string(protein)))
            contig = self._get_conting_from_protein_name(protein)
            if self.contigs_and_sources.has_key(contig):
                for sample, value in self.contigs_and_sources[contig]:
                    sample = sample.split('_') [-1]
                    output_file.write('sample_%s (%s), ' % (sample, value))
            else:
                output_file.write('no_sampl_information')
            output_file.write('\n')
                
        
if __name__ == '__main__': main()
