#!/usr/bin/env python
"""
FUNCTION: Analyses the protein annotation strings and try to put the
          protein into a group based on keywords.

USAGE: PKS_db_analyse_annotation_strings.py <DB_FILE>

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools

def main():
    if not len(sys.argv) == 2:
        print __doc__
        sys.exit(2)

    analyser = AnnotationStringAnalyser(sys.argv[1])
    analyser.analyse_annotation_strings()

class AnnotationStringAnalyser(object):

    """
    Possible groups:
    - PKS
    - Putative_PKS
    - Putative_FAS
    - FAS
    - Hypothetical_protein
    - Acyl_synthase
    - Lipopeptide
    - NRPS
    - Oxidorectuase_dehydrogenase
    - Thioesterase
    - Hydrolase_esterase
    - Amino_acid_adenylation
    - Unknown
    """

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self._set_search_word_lists()

    def analyse_annotation_strings(self):
        self.db_connector.create_db_connection()
        cur = self.db_connector.con.cursor()
        cur.execute('SELECT protein_name, annotation FROM proteins;')
        for protein_name, annotation in cur.fetchall():
            annotation_group = self._search_for_PKS_FAS_keywords(annotation)
            if annotation_group == 'Unknown':
                if self._search_string_set(
                    self.PKS_AB_names, annotation) == True:
                    annotation_group = 'PKS'
                elif self._search_string_set(
                    self.PKS_gene_names, annotation) == True:
                    annotation_group = 'PKS'
                elif self._search_string_set(
                    self.NRPS_gene_names, annotation) == True:
                    annotation_group = 'NRPS'
                elif self._search_string_set(
                    self.NRPS_strings, annotation) == True:
                    annotation_group = 'NRPS'
                elif self._search_string_set(
                    self.NRPS_AB_names, annotation) == True:
                    annotation_group = 'NRPS'
                elif self._search_string_set(
                    self.lipopeptide_names, annotation) == True:
                    annotation_group = 'Lipopeptide'
                elif self._search_string_set(
                    self.oxidoreductase_dehydrogenase_strings,
                    annotation) == True:
                    annotation_group = 'Oxidoreductase_dehydrogenase'
                elif self._search_string_set(
                    self.acyl_synthase_strings, annotation) == True:
                    annotation_group = 'Acyl_synthase'
                elif self._search_string_set(
                    self.thioesterase_strings, annotation) == True:
                    annotation_group = 'Thioesterase'
                elif self._search_string_set(
                    self.hydrolase_esterase_strings, annotation) == True:
                    annotation_group = 'Hydrolase_esterase'
                elif self._search_string_set(
                    self.amino_acid_adenylation_strings, annotation) == True:
                    annotation_group = 'Amino_acid_adenylation'
                elif self._search_string_set(
                    self.hypothetical_protein_strings, annotation) == True:
                    annotation_group = 'Hypothetical_protein'
            self.db_connector.fill_protein_annotation_table(
                protein_name, annotation_group)
        self.db_connector.close_db_connection()

    def _search_for_PKS_FAS_keywords(self, annotation):
        annotation_index = 0
        pks_words = ['Polyketide', 'polyketide', 'POLYKETIDE',
                     'PKS', 'pks', 'Pks']
        fas_words = ['Fab', 'fab', 'FAB', 'Fatty', 'fatty', 'FATTY']
        weakenizers = ['Hypothetical', 'hypothetical', 'HYPOTHETICAL',
                       'Putative', 'putative', 'Probable', 'probable',
                       'similar', 'Similar', 'PREDICTED']
        if annotation.startswith('COG_FAS'):
            return('COG_FAS')
        
        for pks_word in pks_words:
            if annotation_index < 1  and pks_word in annotation:
                annotation_index = 1
        for fas_word in fas_words:
            if annotation_index > -1  and fas_word in annotation:
                annotation_index -= 1
        for weakenizer in weakenizers:
            if weakenizer in annotation and (
               annotation_index == 1 or
               annotation_index == -1):
                annotation_index *= 0.5
        return(self._annotation_index_to_group_name(annotation_index))

    def _search_string_set(self, string_set, annotation):
        for string in string_set:
            if string in annotation:
                return(True)
        return(False)

    def _annotation_index_to_group_name(self, annotation_index):
        if annotation_index == 1:
            return('PKS')
        elif annotation_index == 0.5:
            return('Putative_PKS')
        elif annotation_index == 0:
            return('Unknown')
        elif annotation_index == -0.5:
            return('Putative_FAS')
        elif annotation_index == -1:
            return('FAS')

    def _set_search_word_lists(self):

        self.PKS_AB_names = [
            'Lankamycin', 'Megalomicin', 'Tylactone', 'Protomycinolide',
            'Erythronolide', 'Actinomycin', 'Syringomycin', 'Pristinamycin',
            'ERYTHRONOLIDE','Plipastatin', 'Saframycin', 'Virginiamycin']

        self.PKS_gene_names = [
            'AdmD', 'AdmM', 'AdmO', 'AknB', 'AknC', 'AknF', 'AmphA', 'AmphB',
            'AmphC', 'AmphI', 'AmphJ', 'AmphK',
            'BarC', 'BarE', 'BarF', 'BarG',
            'CtaB', 'CtaC', 'CtaD', 'CtaE', 'CtaF', 'CtaG',
            'CalO5', 'CalE8', 'ChlA2', 'ChlA3', 'ChlA4', 'ChlA5', 'ChlA6',
            'ChlB1','CurA', 'CurC', 'CurF', 'CurG',
            'CurH', 'CurI', 'CurJ', 'CurK', 'CurL', 'CurM',
            'CosB', 'CosC', 'CosF',
            'EpoA', 'EpoB', 'EpoC', 'EpoD', 'EpoD', 'EpoD', 'EpoE', 'EpoF',
            'EryAI', 'EryAII', 'EryAIII', 'EspE'
            'FkbA', 'FkbB', 'FkbC', 'FkbH', 'FkbP', 'FkbQ', 'FscA', 'FscB',
            'FscC', 'FscD', 'FscE', 'FscF', 'FscTE','Fum1p', 
            'GdmAI', 'GdmAII', 'GdmAIII', 'GelA', 'GelB', 'GelC',
            'HbmAI', 'HbmAII', 'HbmAIII', 'HctB', 'HctD', 'HctE', 'HctF',
            'HglC', 'HglD'
            'JamC', 'JamE', 'JamG','JamJ', 'JamK', 'JamL', 'JamM', 'JamN',
            'JamO', 'JamP',
            'MtaB', 'MtaC', 'MtaD', 'MtaE', 'MtaG',
            'MbtB', 'MbtC','MbtD', 'MelB', 'MelC', 'MelD', 'MelE', 'MelF',
            'MelG', 'MerA', 'MerB', 'MerC', 'MerP', 'MmpI', 'MmpII',
            'MmpIII', 'MmpIV', 'MmpV', 'MmpVI',  'MmxB', 'MmxC', 'MtmK',
            'MupE', 'MupG', 'MxaB1', 'MxaB2', 'MxaC', 'MxaD', 'MxaE',
            'MxaF', 'MycA', 'MycB', 'MycC',
            'NanA1', 'NanA2','NanA3', 'NanA4', 'NanA5', 'NanA6', 'NanA7',
            'NanA8', 'NanA9', 'NanA10', 'NanA11', 'NbmB', 
            'NigAI', 'NigAII', 'NigAIII', 'NigAIV', 'NigAIX', 'NigAV', 'NigAVI',
            'NigAVII', 'NigAX', 'NigAXI',
            'NosA', 'NosB', 'NosC', 'NosD', 'NovH', 'NovJ', 'NrpS',
            'NysA', 'NysB', 'NysC', 'NysE', 'NysI', 'NysJ', 'NysK',
            'ObsA', 'ObsB', 'ObsC', 'OnnB', 'OnnI', 'OxyB', 'OxyP',
            'PedM','PimI','Plm1', 'Plm2-3', 'Plm4', 'Plm5', 'Plm6',
            'Plm7', 'Plm8', 'PlmT7',
            'RemA', 'RemB', 'RemE', 'RifA', 'RifB', 'RifD', 'RifE','RifR',
            'RimA', 'RimB', 'RimJ', 
            'SalA', 'SalB','ShnA', 'ShnB', 'ShnC', 'ShnD', 'ShnE', 'ShnG',
            'StiA', 'StiB', 'StiC', 'StiD', 'StiE', 'StiF', 'StiG' 'StiH', 'StiJ'
            'YbtT']

        self.NRPS_gene_names = [
            'BmyA', 'BmyB', 'BmyC', 'BmyD',
            'BryA', 'BryB', 'BryC', 'BryD', 'BryP', 'BryQ', 'BryX',
            'CesA', 'CesB', 'DhbF',
            'FenA', 'FenB', 'FenC', 'FenD', 'FenE', 'FenF',
            'FxbB', 'FxbC', 'MbtE', 'MbtF',
            'NdaB', 'NdaC', 'NdaD', 'NdaF',
            'PvdD', 'PvdG', 'PvdI', 'PvdJ', 'PvdL']

        self.NRPS_AB_names = [
            'Indigoidine', 'Cyclosporine', 'Pyoverdine', 'Microcystin',
            'Bacitracin', 'Enterobactin', 'Plipastatin', 'Saframycin',
            'Gramicidin', 'gramicidin', 'Tyrocidine']

        self.NRPS_strings = [
            'non-ribosomal', 'Non-ribosomal', 'non ribosomal', 'Non ribosomal',
            'Nonribosomal', 'nonribosomal', 'no-ribosomal', 'NON-RIBOSOMAL',
            'NRPS', 'Peptide synthetase', 'Non-ribosimal', 'peptide synthetase',
            'peptide synthase']

        self.lipopeptide_names = ['BamD', 'MassA', 'MassB', 'MassC']

        self.oxidoreductase_dehydrogenase_strings = [
            'dehydrogenase', 'Dehydrogenase', 'oxidoreductase',
            'Oxidoreductase', 'OXIDOREDUCTASE', 'Quinone reductase',
            'quinone reductase']

        self.acyl_synthase_strings = [
            'ketoacyl synthase', 'Ketosynthase', 'Ketoacyl synthase',
            '3-oxoacyl-[acyl-carrier-protein] synthase']

        self.hypothetical_protein_strings = [
            'Hypothetical protein', 'hypothetical protein',
            'Hypothetical Protein', 'Predicted protein',
            'PREDICTED', 'Novel protein']

        self.thioesterase_strings = [
            'Thioesterase', 'thioesterase']

        self.hydrolase_esterase_strings = [
            'hydrolase', 'esterase', 'Esterase', 'Hydrolase']

        self.amino_acid_adenylation_strings = ['Amino acid adenylation']

        # Not used so far
        self.misc_gene_names = [
            'StaA', 'StaB','StaC','StaD',
            'Aft9-1',  'ChlA1',  'BioH',
            'DszA', 'DszB','DszC', 'DszD', 
            'IgiD', 
            'McyB', 'McyC', 'McyD', 'McyE', 'McyG',
            'NapA',  'NapB', 'NapC', 'NapD', 'NapE',
            'NcpA', 'NcpB',
            'NikP1', 'NikP2',
            'NlmA1', 'NlmA2', 'NlmA3', 'NlmA4', 'NlmA5', 'NlmA6','NlmA7'
            'NocA', 'NocB', 'OlmA2',
            'PchC','PimS', 'PskE',
            'PstA', 'PstB', 'PstD',            
            'Rimo', 'RkpH', 'RkpA',
            'SubA', 'SubB', 'SubC',
            'TubB', 'TubC', 'TubD', 'TubE', 'TubF',
            'VenA', 'YoxD', 'ChlK', 'CylH','CylI']

        # 'Iturin' - lipopeptide antibiotic
        # 'Bacillomycin' - lipopeptide antibiotic
        # 'Lichenysin' - lipopeptide antibiotic
        # Amphisin - lipopeptide antibiotic
        # Fengycin - lipopeptide antibiotic
        # Nikkomycins - nucleoside analogs of UDP-N-acetylglucosamine
        # 'Arthrofactin' - lipopeptide antibiotic
        # 'Syringopeptin'  - lipodepsipeptide
        # 'Daptomycin' - lipodepsipeptide
        # 'Enniatin' - hexadepsipeptide
        # 'Thaxtomin' - dipeptide
        # Pyochelin - peptide?
        # Syringomycin - lipopeptide
        # Phenyloxazoline' - ?
        # Mycocerosate', ???

if __name__ == '__main__': main()
