#!/usr/bin/env python
"""
FUNCTION: Compares the occurance of the AT and KS domain.

USAGE:  PKS_db_create_AT_KS_UniRef_comparison.py <DB_FILE> 
         
Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools


def main():
    if not len(sys.argv) == 3:
        print __doc__
        sys.exit(2)
    protein_overview_creator = ProteinOverviewCreator(sys.argv[1])
    protein_overview_creator.select_PKS_protein(sys.argv[2])
    protein_overview_creator.analyse_species_of_single_hits_protein()
    protein_overview_creator.analyse_multi_hits_protein()
    
class ProteinOverviewCreator(object):

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        self.cur = self.con.cursor()

    def select_PKS_protein(self, cut_off_GPHS_value):
        self.mult_hit_proteins_sorted_by_GPHS = []
        self.single_hit_PKS_proteins = []
        for protein in self.db_connector.get_all_used_proteins():
            # DEVELOPMENT ONLY
            #if not protein.startswith('UniRef100_P'): continue
            GPHS = self.db_connector.calc_GPHS(protein)
            # For protein with more than one domain hit the GPHS
            # is not None
            if GPHS >= float(cut_off_GPHS_value):
                self.mult_hit_proteins_sorted_by_GPHS.append([GPHS, protein])
            # For the single domain hit protein the GPHS is None;
            # then the single hit is used to discriminate
            elif GPHS == None:
                hits = self.db_connector.get_all_hits_of_protein(protein)
                # Just a as control
                if len(hits) > 1:
                    print 'Something is fishy here. The protein "%s" should ' \
                          'have a GPHS!' % protein
                    continue
                elif len(hits) == 0:
                    print 'Something is fishy here. The protein "%s" should ' \
                          'have a hit!' % protein
                    continue
                annnotation = self.db_connector.get_tree_based_annotation_of_hit(
                    hits[0])
                if annnotation == 'PKS':
                    self.single_hit_PKS_proteins.append(protein)
        self.mult_hit_proteins_sorted_by_GPHS.sort()
        self.mult_hit_proteins_sorted_by_GPHS.reverse()

    def analyse_species_of_single_hits_protein(self):
        """Group the single hit protein of species-wise together

        Count which organism have AT and/or KS.
        """
        output_file = open('AT_KS_single_hits_species_wise.txt', 'w')
        self.species_and_hit_proteins = {}
        self.species_and_hit_domains = {}
        for protein in self.single_hit_PKS_proteins:
            species = self.db_connector.get_species_of_protein(protein)
            if self.species_and_hit_proteins.has_key(species):
                self.species_and_hit_proteins[species].append(protein)
                self.species_and_hit_domains[species].append(
                    self.db_connector.build_domain_composition_string(protein))
            else:
                self.species_and_hit_proteins[species] = [protein]
                self.species_and_hit_domains[species] = [
                    self.db_connector.build_domain_composition_string(protein)]
        counter_AT_but_not_KS = 0
        counter_KS_but_not_AT = 0
        counter_KS_and_AT = 0
        counter_neither_AT_nor_KS = 0
        for species in self.species_and_hit_domains.keys():
            domain_collec = self.species_and_hit_domains[species]
            output_file.write('%s %s\n'  % (species, ' + '.join(
                self.species_and_hit_domains[species])))
            if 'AT(PKS)' in domain_collec and not 'KS(PKS)' in domain_collec:
                counter_AT_but_not_KS += 1
            elif 'KS(PKS)' in domain_collec and not 'AT(PKS)' in domain_collec:
                counter_KS_but_not_AT += 1
            elif 'AT(PKS)' in domain_collec and 'KS(PKS)' in domain_collec:
                counter_KS_and_AT += 1
            elif not 'AT(PKS)' in domain_collec and not 'KS(PKS)' in domain_collec:
                counter_neither_AT_nor_KS += 1
        output_file.write(
            '--------------------------------\n'
            'Total number of single hits: %s\n'
            'Species with AT but not KS single hits: %s\n'
            'Species with KS but not AT single hits: %s\n'
            'Species with AT + KS single hits: %s\n'
            'Species with no AT and not KS single hits: %s\n'
            '--------------------------------\n' % (
            len(self.single_hit_PKS_proteins), counter_AT_but_not_KS,
            counter_KS_but_not_AT, counter_KS_and_AT,
            counter_neither_AT_nor_KS))
        output_file.close()

    def analyse_multi_hits_protein(self):
        output_file = open('AT_KS_analysis_of_multi_hit_proteins.txt', 'w')
        counter_AT_but_not_KS = 0
        counter_KS_but_not_AT = 0
        counter_KS_and_AT = 0
        counter_neither_AT_nor_KS = 0
        for GPHS, protein in self.mult_hit_proteins_sorted_by_GPHS:
            domain_struc = self.db_connector.build_domain_composition_string(
                protein)
            flag = None
            if 'AT(PKS)' in domain_struc and not 'KS(PKS)' in domain_struc:
                flag = 'AT_only'
                counter_AT_but_not_KS += 1
            elif 'KS(PKS)' in domain_struc and not 'AT(PKS)' in domain_struc:
                flag = 'KS_only'
                counter_KS_but_not_AT += 1
            elif 'AT(PKS)' in domain_struc and 'KS(PKS)' in domain_struc:
                flag = 'AT_and_KS'
                counter_KS_and_AT += 1
            elif not 'AT(PKS)' in domain_struc and not 'KS(PKS)' in domain_struc:
                flag = 'No_AT_and_No_KS'
                counter_neither_AT_nor_KS += 1
            output_file.write('%s: %s %s\n' % (protein, domain_struc, flag))
            
        output_file.write(
            '--------------------------------\n'
            'AT but not KS: %s\n'
            'KS but not AT: %s\n'
            'AT + KS: %s\n'
            'No AT and not KS: %s\n'
            '--------------------------------' % (
            counter_AT_but_not_KS, counter_KS_but_not_AT, counter_KS_and_AT,
            counter_neither_AT_nor_KS))

    def _write_protein_string(self, protein, GPHS, output_file):
        species = self.db_connector.get_species_of_protein(protein)
        taxonomy = self.db_connector.get_full_tax_of_species(species)
        output_file.write(
            '%s\t\'%s\'\t%s\t%s\t%s\t%s\t%s\t%s\n' % (
            protein,
            GPHS,
            self.db_connector.build_domain_composition_string(protein),
            self.db_connector.get_annotation_string_of_protein(protein),
            self.db_connector.get_text_string_annotation_group_of_protein(
            protein),
            species,
            taxonomy,
            self.db_connector.get_source_of_protein(protein)))
        

if __name__ == '__main__': main()
