#!/usr/bin/env python

"""
FUNCTION: Uses the database to create a CSV file containing overview
          table of all PKS HMM hits and their features.

USAGE: PKS_db_create_overview_CSV_file.py <DB_FILE> [<OUTPUT_FILE_NAME>]
         
Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools

def main():
    if not (len(sys.argv) == 2 or len(sys.argv) == 3):
        print __doc__
        sys.exit(2)

    output_file_name = 'PKS_db_hit_overview.csv'
    if len(sys.argv) == 3:
        output_file_name = sys.argv[2]
    creator = CVSOverviewCreator(sys.argv[1])
    creator.build_CVS_overview_file(output_file_name)

class CVSOverviewCreator(object):

    proteins_and_domain_compositions = {}

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            self.db_connector.create_db_file()
            
    def build_CVS_overview_file(self, output_file_name):
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        domains = self._get_domain_names()
        domains.sort()
        self.output_file = open(output_file_name, 'w')
        self._write_table_header_to_file()
        for domain in domains:
            self._add_domain_hits(domain)
        self.output_file.close()
        self.db_connector.close_db_connection()
        print 'Wrote results to "%s".' % (output_file_name)

    def _write_table_header_to_file(self):
        self.output_file.write(
            '%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t'
            '%s\t%s\t%s\t%s\n' % (
            'hit name', 'HMM name', 'tree position', 'number header',
            'added by cluster', 'e-value', 'bitscore', 'hit length',
            'hit start position', 'hit end position', 'hit protein name',
            'hit protein domain structure', 'hit protein annotation',
            'hit protein length', 'source', 'species',
            'annotation group', 'taxonomy', 'annotation by tree position',
            'comments'))        

    def _add_domain_hits(self, domain):
        tree_positions_and_domain_HMM_hit_name = []
        cur = self.con.cursor()
        cur.execute(
            # The 'added_by_cluster=""' selects only hits that are
            # really in the tree and not only represented by others.
            # The other members of a cluster are added with the hits
            # below.
            'SELECT domain_HMM_hit_name, HMM_hit_name FROM HMM_hits '
            'WHERE HMM_domain_name="%s" AND added_by_cluster_of="-";' % (domain))
        for domain_HMM_hit_name, HMM_hit_name in cur.fetchall():
            cur.execute(
                'SELECT tree_position FROM hit_tree_position '
                'WHERE domain_HMM_hit_name="%s" ;' % (domain_HMM_hit_name))
            try:
                tree_position = cur.fetchone()[0]
                tree_positions_and_domain_HMM_hit_name.append(
                    [tree_position, domain_HMM_hit_name, HMM_hit_name])
            except:
                print 'There seems to be not tree position for "%s".' % (
                    domain_HMM_hit_name)
                pass
        if len(tree_positions_and_domain_HMM_hit_name) == 0:
            print 'No tree position for domain "%s". Seems as they still have '\
                  'to be added to the database.'  % domain
        tree_positions_and_domain_HMM_hit_name.sort()
        for tree_position, domain_HMM_hit_name, HMM_hit_name in \
                tree_positions_and_domain_HMM_hit_name:
            self._get_features_and_write_to_output(
                domain_HMM_hit_name, HMM_hit_name, domain, tree_position)
            self._add_cluster_members_of_hit(
                HMM_hit_name, domain, tree_position)
                    
        cur.close()

    def _add_cluster_members_of_hit(
        self, HMM_hit_name, domain, tree_position):
        cur = self.con.cursor()
        cur.execute('SELECT domain_HMM_hit_name, HMM_hit_name FROM HMM_hits '
                    'WHERE added_by_cluster_of="%s";' % (
            HMM_hit_name))
        rows = cur.fetchall()
        if not len(rows) == 0:
            counter = 1
            for row in rows:
                cluster_member_domain_HMM_hit_name = row[0]
                cluster_member_HMM_hit_name = row[1]
                cluster_member_tree_position = "%s.%s" % (tree_position, counter)
                self._get_features_and_write_to_output(
                    cluster_member_domain_HMM_hit_name,
                    cluster_member_HMM_hit_name, domain, tree_position)

    def _get_features_and_write_to_output(
        self, domain_HMM_hit_name, HMM_hit_name, domain, tree_position):
        hit_features = self.db_connector.get_hit_features(domain_HMM_hit_name)
        protein_features = self._get_protein_features(
            hit_features['hit_protein_name'])
        species_tax_string = self._get_species_tax_string(
            protein_features['species'])
        protein_annotation_group = self.db_connector.get_protein_annotation(
            hit_features['hit_protein_name'])
        hit_anno_by_tree_postion, commments = self._get_hit_anno_by_tree_pos(
            domain_HMM_hit_name)

        if not self.proteins_and_domain_compositions.has_key(
            hit_features['hit_protein_name']):
            domain_composition = (
                self.db_connector.build_domain_composition_string(
                hit_features['hit_protein_name']))
            self.proteins_and_domain_compositions[
                hit_features['hit_protein_name']] = domain_composition

        self.output_file.write(
            '%s\t%s\t%s\t%s\t%s\t"%s"\t"%s"\t%s\t%s\t%s\t"%s"\t%s\t%s\t%s\t%s'
            '\t%s\t%s\t%s\t%s\t%s\n' % (
            HMM_hit_name,
            domain,
            tree_position,
            hit_features['simplified_hit_header'],
            hit_features['added_by_cluster_of'],
            hit_features['e_value'],
            hit_features['bitscore'],
            hit_features['hit_length'],
            hit_features['hit_start_position'],
            hit_features['hit_end_position'],
            hit_features['hit_protein_name'],
            self.proteins_and_domain_compositions[hit_features[
            'hit_protein_name']],
            protein_features['annotation'],
            protein_features['length'],
            protein_features['source'],
            protein_features['species'],
            protein_annotation_group,
            species_tax_string,
            hit_anno_by_tree_postion,
            commments))

    def _get_tree_based_annotation_group(self, domain_HMM_hit_name):
        cur = self.con.cursor()
        cur.execute(
            'SELECT tree_based_annotation_group FROM '
            'tree_based_hit_annotations WHERE domain_HMM_hit_name="%s";' % (
            domain_HMM_hit_name))
        tree_based_annotation_group = cur.fetchone()
        if not tree_based_annotation_group == None:
            return(tree_based_annotation_group[0])
        else:
            return('-')

    def _get_hit_anno_by_tree_pos(self, domain_HMM_hit_name):
        hit_annotation = '-'
        comments = '-'
        cur = self.con.cursor()
        cur.execute('SELECT tree_based_annotation_group, comments FROM '
                    'tree_based_hit_annotations WHERE '
                    'domain_HMM_hit_name="%s";' % (domain_HMM_hit_name))
        row = cur.fetchone()
        cur.close()
        if not row == None:
            hit_annotation = row[0]
            comments = row[1]
        return(hit_annotation, comments)

    def _get_protein_features(self, hit_protein_name):
        protein_features = {}
        cur = self.con.cursor()
        cur.execute('SELECT length, annotation, species, source FROM proteins '
                    'WHERE protein_name="%s";' % (hit_protein_name))
        row = cur.fetchone()
        cur.close()
        if not row == None:
            (protein_features['length'],
             protein_features['annotation'],
             protein_features['species'],
             protein_features['source']) = row
        else:
            (protein_features['length'], 
             protein_features['annotation'],
             protein_features['species'],
             protein_features['source']) = ['-','-','-','-']
        return(protein_features)

    def _get_species_tax_string(self, species):
        tax_string = '-'
        cur = self.con.cursor()
        cur.execute('SELECT full_taxonomie_string FROM species '
                    'WHERE species_name="%s";' % (species))
        row = cur.fetchone()
        cur.close()
        if not row == None:
            tax_string = row[0]
        return(tax_string)
        
    def _get_domain_names(self):
        domains = []
        cur = self.con.cursor()
        cur.execute('SELECT DISTINCT HMM_domain_name FROM HMM_hits;')

        for row in cur.fetchall():
            domains.append(row[0])
        cur.close()
        return(domains)

if __name__ == '__main__': main()
