#!/usr/bin/env python
"""
FUNCTION: Checks if the quality of the proteins of all hits in a fasta
          file by usesing the PKS_db. Hit with protein that are in the
          db are marked. In the bottom of the file, proteins that have
          hits of that domain in the database but not in the fasta
          file are listed.

USAGE: PKS_db_create_new_HMM_search_comparison.py <DB_FILE> <FASTA_FILE> <DOMAIN>
         
Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools
import fastaparser

def main():
    if not len(sys.argv) == 4:
        print __doc__
        sys.exit(2)

    comparator = HMMSearchComparator(sys.argv[1])
    comparator.get_hits_from_fasta_file(sys.argv[2])
    comparator.create_list_of_proteins_with_PKS_marked_hits_of_domain(
        sys.argv[3])
    comparator.compare_new_and_old_hits()
        
class HMMSearchComparator(object):

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        self.cur = self.con.cursor()

    def get_hits_from_fasta_file(self, fasta_file):
        # List of dictionaries
        self.hit_protein_with_features = []
        fasta_parser = fastaparser.FastaParser()
        for header, seq in fasta_parser.parse_fasta_file(fasta_file):
            self.hit_protein_with_features.append(
                self._extract_header_features(header))

    def _extract_header_features(self, header):
        protein = header.split('_from_')[0]
        bitscore = header.split('bitscore: ')[1].split(', e-value')[0]
        evalue = header.split('e-value: ')[1].split(', length:')[0]
        return({'protein' : protein, 'bitscore' : bitscore,
                'evalue' : evalue})

    def create_list_of_proteins_with_PKS_marked_hits_of_domain(self, domain):
        self.wanted_PKS_proteins = []
        for hit in self.db_connector.get_all_hits_of_domain(domain):
            # Only UniRef hits are needed
            if not '_UniRef' in hit: continue
            tree_based_anno = self.db_connector.get_tree_based_annotation_of_hit(
                hit)
            # Use only PKS hits and skip others
            if not tree_based_anno == 'PKS': continue
            protein = self.db_connector.get_protein_of_a_hit(hit)
            self.wanted_PKS_proteins.append(protein)
        # Make list non-redundant
        self.wanted_PKS_proteins = list(set(self.wanted_PKS_proteins))

    def compare_new_and_old_hits(self):
        wanted_PKS_proteins_tick_off = self.wanted_PKS_proteins[:]
        print '# Protein_name hit_biscore, hit_e-value, mark, GPHS, domain_string'
        for protein_and_feature in self.hit_protein_with_features:
            protein = protein_and_feature['protein']
            bitscore = protein_and_feature['bitscore']
            evalue = protein_and_feature['evalue']
            print protein, bitscore, evalue,
            if protein in self.wanted_PKS_proteins:
                domain_string = self.db_connector.build_domain_composition_string(
                    protein)
                if len(domain_string) > 50:
                    domain_string = domain_string[:50] + '...'
                print 'X', self.db_connector.calc_GPHS(protein),
                print domain_string
            else:
                print ''
            if protein in wanted_PKS_proteins_tick_off:
                wanted_PKS_proteins_tick_off.remove(protein)

        # Print missing proteins sorted by GPHS
        GPHS_and_proteins = []
        print '\nMissing proteins (found in the old search not the new one):'
        for protein in wanted_PKS_proteins_tick_off:
            GPHS = self.db_connector.calc_GPHS(protein)
            GPHS_and_proteins.append([GPHS, protein])
        GPHS_and_proteins.sort()
        GPHS_and_proteins.reverse()
        for GPHS, protein in GPHS_and_proteins:
            print protein, GPHS, self.db_connector.build_domain_composition_string(
                protein)

if __name__ == '__main__': main()
