#!/usr/bin/env python
"""
FUNCTION: Finds organisms that contain proteins that are not reported
          to be PKS. Two files are produced: One containing the
          anylsis of all used organism and one that only show the one
          that seem to be unknown to produce PKS.

USAGE: PKS_db_find_species_without_tax_string.py <DB_FILE> <MIN_GPHS>

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import kuffileproc
import pks_db_tools


def main():
    if not len(sys.argv) == 3:
        print __doc__
        sys.exit(2)
    hunter = NewPKSProdocerHunter(sys.argv[1])
    hunter.hunt_new_PKS_producers(sys.argv[2])

class NewPKSProdocerHunter(object):

    species_without_tax_string = []

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()

    def hunt_new_PKS_producers(self, min_GPHS):
        self.species_and_hit_proteins = {}
        fileproc = kuffileproc.FileProcessor()
        output_file_pure = fileproc.get_file('PKS_producer_only_new.txt', 'w')
        output_file_all = fileproc.get_file('PKS_producer_all.txt', 'w')
        self.min_GPHS = min_GPHS
        for species in self.db_connector.get_all_used_species():
            if species == '' or species == 'unknown': continue
            self.output_string = ''
            self.known_PKS_found = False
            self.new_multi_domain_PKS_found = False
            self.new_single_domain_PKS_found = False
            for protein in self.db_connector.get_all_proteins_of_species(species):
                self._anaylse_protein(protein)
            # Evaluate protein analyses
            if (self.known_PKS_found == False and
                self.new_multi_domain_PKS_found == True and
                self.new_single_domain_PKS_found == True):
                self.output_string = '* %s %s\n%s\n'  % (
                    species,
                    '==> NEW: multi and single domain hit protein',
                    self.output_string)
                output_file_pure.write(self.output_string)
            elif (self.known_PKS_found == False and
                  self.new_multi_domain_PKS_found == True):
                self.output_string = '* %s %s\n%s\n'  % (
                    species,
                    '==> NEW: multi domain hit protein',
                    self.output_string)
                output_file_pure.write(self.output_string)
            elif (self.known_PKS_found == False and
                  self.new_single_domain_PKS_found == True):
                self.output_string = '* %s %s\n%s\n'  % (
                    species,
                    '==> NEW single domain hit protein',
                    self.output_string)
                output_file_pure.write(self.output_string)
            else:
                self.output_string = '* %s %s\n%s\n'  % (
                    species,
                    '==> ONLY OLD protein',
                    self.output_string)
            output_file_all.write(self.output_string)
        output_file_pure.close()
        output_file_all.close()

    def _anaylse_protein(self, protein):
        annotation_group = self.db_connector.get_protein_annotation(protein)
        GPHS = self.db_connector.calc_GPHS(protein)
        protein_is_a_PKS = False
        # Known PKS proteins
        if annotation_group == 'PKS':
            self.known_PKS_found = True
            protein_is_a_PKS = True
        # New mulit domain hit protein
        elif GPHS >= float(self.min_GPHS):
            self.new_multi_domain_PKS_found = True
            protein_is_a_PKS = True
        # New single domain hit protein            
        elif GPHS == None:
            hits = self.db_connector.get_all_hits_of_protein(protein)
            if len(hits) != 1:
                print ('Error: Something is fishy with protein "%s". '
                       'It should have exactly one domain hit.' % (protein))
                return()
            if self.db_connector.get_tree_based_annotation_of_hit(
                hits[0]) == 'PKS':
                self.new_single_domain_PKS_found = True
                protein_is_a_PKS = True
        if protein_is_a_PKS == True:
            self.output_string += ' - %s %s %s  [%s]\n' % (
                protein, annotation_group, GPHS,
                self.db_connector.build_domain_composition_string(protein))

if __name__ == '__main__': main()
