#!/usr/bin/env python
"""
FUNCTION: For each domain the average of the global protein hit score
          (GPHS) is calculated. This score reflects the likekelyhood
          that a hit protein is really a PKS hit. It is calculated by
          substracting the number of other hits from the number of PKS
          domain hits of a protein and dividing this by the total
          number of hits in the protein. A value between 0 and 1
          implies that the protein is with higher likelyhood a PKS, a
          values between 0 and -1 implies that the proteins with
          higher likelyhood a non-PKS protein. The values are only
          calculated if there are two or more domain hits found in a
          protein.
          
USAGE: PKS_db_create_global_protein_hits_statistics.py <DB_FILE>
         
Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools

def main():
    if not len(sys.argv) == 2:
        print __doc__
        sys.exit(2)
    GPHS_stat_creator = GPHSStatCreator(sys.argv[1])
    GPHS_stat_creator.calc_average_GPHSs_for_all_domains()
    
class GPHSStatCreator(object):

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        self.cur = self.con.cursor()

    def calc_average_GPHSs_for_all_domains(self):
        for domain in self.db_connector.get_all_used_domains():
            # Build a non-redundant list of proteins that are hit by
            # the domain
            proteins_that_are_hit_by_domain = []
            for hit in self.db_connector.get_all_hits_of_domain(domain):
                hit_protein = self.db_connector.get_protein_of_a_hit(hit)
                proteins_that_are_hit_by_domain.append(hit_protein)
            non_reduntant_protein_list = list(
                set(proteins_that_are_hit_by_domain))
            # Sum up all GPHS of the proteins
            sum_of_GPHSs = 0
            protein_with_valid_GPHS = 0
            for hit_protein in non_reduntant_protein_list:
                GPHS = self.db_connector.calc_GPHS(hit_protein)
                if not GPHS == None:
                    sum_of_GPHSs += GPHS
                    protein_with_valid_GPHS += 1
            # Print output
            print domain,
            print float(sum_of_GPHSs)/protein_with_valid_GPHS

if __name__ == '__main__': main()
